package auth //nolint:testpackage

import (
	"testing"
	"time"

	"github.com/stretchr/testify/assert"
	"github.com/stretchr/testify/require"
	"gopkg.in/dgrijalva/jwt-go.v3"
	"orus.io/orus-io/go-orusapi/database"

	"orus.io/orus-io/rednerd/migration"
	"orus.io/orus-io/rednerd/models"
	"orus.io/orus-io/rednerd/testutils"
)

func TestAuth(t *testing.T) {
	log := testutils.GetLogger(t)
	ctx, cancel := testutils.GetContext(t)
	ctx = log.WithContext(ctx)

	defer cancel()

	testdb := database.GetTestDB(ctx, t, migration.Source)
	defer testdb.Close()

	t.Run("basic", func(t *testing.T) {
		auth := NewAuth(ctx, TokenOptions{
			Secret:          "test",
			Expiration:      time.Minute,
			CacheMaxSize:    10,
			CachePurgeDelay: time.Minute,
		}, testdb.DB, log)
		defer auth.Close()

		for _, id := range []string{
			"1", "2", "3",
			"4", "5", "6",
			"7", "8", "9",
			"10", "11", "12",
		} {
			_, _, err := auth.MakeToken(models.Principal{
				StandardClaims: jwt.StandardClaims{
					Subject: id,
				},
			}, false)
			require.NoError(t, err)
		}

		assert.Equal(t, 10, auth.cache.Len())
	})
	t.Run("no-secret", func(t *testing.T) {
		auth := NewAuth(ctx, TokenOptions{
			Expiration:      time.Minute,
			CacheMaxSize:    10,
			CachePurgeDelay: time.Minute,
		}, testdb.DB, log)
		defer auth.Close()
	})
}
